@@ -1,4 +1,5 @@
 import math
+from typing import List
 from tqdm import tqdm
 from inspect import isfunction
 from functools import partial, wraps
@@ -21,6 +22,8 @@ import kornia.augmentation as K
 
 from resize_right import resize
 
+from imagen_pytorch.t5 import t5_encode_text, T5_SMALL_EMBED_DIM
+
 # constants
 
 NAT = 1. / math.log(2.)
@@ -98,7 +101,6 @@ def resize_image_to(image, target_image_size):
 
 # image normalization functions
 # ddpms expect images to be in the range of -1 to 1
-# but CLIP may otherwise
 
 def normalize_neg_one_to_one(img):
     return img * 2 - 1
@@ -694,7 +696,7 @@ class Unet(nn.Module):
         dim,
         *,
         image_embed_dim = None,
-        text_embed_dim = None,
+        text_embed_dim = T5_SMALL_EMBED_DIM,
         cond_dim = None,
         num_image_tokens = 4,
         num_time_tokens = 2,
@@ -1025,7 +1027,6 @@ class Imagen(BaseGaussianDiffusion):
         self,
         unet,
         *,
-        clip = None,
         image_size = None,
         channels = 3,
         timesteps = 1000,
@@ -1062,7 +1063,7 @@ class Imagen(BaseGaussianDiffusion):
             assert exists(image_size) ^ exists(image_sizes), 'only one of image_size or image_sizes must be given'
             image_size = default(image_size, lambda: image_sizes[-1])
         else:
-            raise Error('either image_size, image_sizes, or clip must be given to decoder')
+            raise Error('either image_size or image sizes must be given to imagen')
 
         # channels
 
@@ -1165,7 +1166,7 @@ class Imagen(BaseGaussianDiffusion):
             unet.to(device)
 
     def p_mean_variance(self, unet, x, t, text_embeds = None, text_mask = None, lowres_cond_img = None, clip_denoised = True, predict_x_start = False, learned_variance = False, cond_scale = 1., model_output = None):
-        assert not (cond_scale != 1. and not self.can_classifier_guidance), 'the decoder was not trained with conditional dropout, and thus one cannot use classifier free guidance (cond_scale anything other than 1)'
+        assert not (cond_scale != 1. and not self.can_classifier_guidance), 'imagen was not trained with conditional dropout, and thus one cannot use classifier free guidance (cond_scale anything other than 1)'
 
         pred = default(model_output, lambda: unet.forward_with_cond_scale(x, t, text_embeds = text_embeds, text_mask = text_mask, cond_scale = cond_scale, lowres_cond_img = lowres_cond_img))
 
@@ -1297,21 +1298,24 @@ class Imagen(BaseGaussianDiffusion):
     @eval_decorator
     def sample(
         self,
-        text = None,
+        texts: List[str] = None,
         text_mask = None,
         text_embeds = None,
         batch_size = 1,
         cond_scale = 1.,
         stop_at_unet_number = None
     ):
+        device = next(self.parameters()).device
+
+        if exists(texts) and not exists(text_embeds) and not self.unconditional:
+            text_embeds = t5_encode_text(texts)
+            text_embeds.to(device)
+
         if not self.unconditional:
             batch_size = text_embeds.shape[0]
 
-        if exists(text) and not exists(text_embeds) and not self.unconditional:
-            assert False, 'needs to be built'
-
-        assert not (self.condition_on_text and not exists(text_embeds)), 'text or text encodings must be passed into decoder if specified'
-        assert not (not self.condition_on_text and exists(text_embeds)), 'decoder specified not to be conditioned on text, yet it is presented'
+        assert not (self.condition_on_text and not exists(text_embeds)), 'text or text encodings must be passed into imagen if specified'
+        assert not (not self.condition_on_text and exists(text_embeds)), 'imagen specified not to be conditioned on text, yet it is presented'
 
         img = None
         is_cuda = next(self.parameters()).is_cuda
@@ -1349,7 +1353,7 @@ class Imagen(BaseGaussianDiffusion):
     def forward(
         self,
         image,
-        text = None,
+        texts: List[str] = None,
         text_embeds = None,
         text_mask = None,
         unet_number = None
@@ -1357,7 +1361,7 @@ class Imagen(BaseGaussianDiffusion):
         assert not (len(self.unets) > 1 and not exists(unet_number)), f'you must specify which unet you want trained, from a range of 1 to {len(self.unets)}, if you are training cascading DDPM (multiple unets)'
         unet_number = default(unet_number, 1)
         unet_index = unet_number - 1
-
+        
         unet = self.get_unet(unet_number)
 
         target_image_size   = self.image_sizes[unet_index]
@@ -1371,8 +1375,9 @@ class Imagen(BaseGaussianDiffusion):
 
         times = torch.randint(0, self.num_timesteps, (b,), device = device, dtype = torch.long)
 
-        if exists(text) and not exists(text_embeds) and not self.unconditional:
-            assert False, 'not built yet'
+        if exists(texts) and not exists(text_embeds) and not self.unconditional:
+            text_embeds = t5_encode_text(texts)
+            text_embds.to(image.device)
 
         assert not (self.condition_on_text and not exists(text_embeds)), 'text or text encodings must be passed into decoder if specified'
         assert not (not self.condition_on_text and exists(text_embeds)), 'decoder specified not to be conditioned on text, yet it is presented'
